dnl ---------------------------------------------------------------------------
dnl
dnl  particles.c.m4
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  Copyright (C) 2007 Takashi Uneyama
dnl
dnl  This program is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation; either version 2, or (at your option)
dnl  any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
dnl  USA.
dnl
dnl ---------------------------------------------------------------------------
include(`dimension.m4')`'dnl
GENERATE_HEADER(celllist.c,
                create or update the linked cell list.)

#include "common.h"

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "xmalloc.h"
#include "vector.h"
#include "celllist.h"

#define R_CUT 2.0

/*---------------------------------------------------------------------------*/
PROTOTYPE_FUNCTIONS(`dnl
static linked_cell_list *allocate_linked_cell_list@D@(soft_particles@D@ *particles,
                                                    box_geometry *geometry);
static void clear_linked_cell_list@D@(linked_cell_list *celllist,
                                    soft_particles@D@ *particles);
static void construct_linked_cell_list@D@(linked_cell_list *celllist,
                                        vector@D@ **r,
                                        soft_particles@D@ *particles,
                                        box_geometry *geometry);'`')

/*---------------------------------------------------------------------------*/
linked_cell_list *allocate_linked_cell_list(soft_particles *particles,
                                            box_geometry *geometry)
{
    linked_cell_list *celllist;

    SWITCH_FUNCTION(particles->dimension,
                    celllist = allocate_linked_cell_list@D@(&(particles->particles@D@),geometry);,
                    celllist = NULL;)

    return celllist;
}

/*---------------------------------------------------------------------------*/
void free_linked_cell_list(linked_cell_list *celllist)
{
    if(celllist == NULL)
        return;

    free(celllist->head[0]);
    free(celllist->list[0]);
    free(celllist->head);
    free(celllist->list);
    free(celllist);
}

/*---------------------------------------------------------------------------*/
void construct_linked_cell_list(linked_cell_list *celllist,
                                soft_particles *particles,
                                box_geometry *geometry)
{
    SWITCH_FUNCTION(particles->dimension,
                    construct_linked_cell_list@D@(celllist,particles->particles@D@.r,
                                                  &(particles->particles@D@),geometry);)
}

/*---------------------------------------------------------------------------*/
void construct_linked_cell_list_star(linked_cell_list *celllist,
                                     soft_particles *particles,
                                     box_geometry *geometry)
{
    SWITCH_FUNCTION(particles->dimension,
                    construct_linked_cell_list@D@(celllist,particles->particles@D@.r_star,
                                                  &(particles->particles@D@),geometry);)
}

/*---------------------------------------------------------------------------*/
EXPAND_FUNCTIONS(`dnl
linked_cell_list *allocate_linked_cell_list@D@(soft_particles@D@ *particles,
                                               box_geometry *geometry)
{
    linked_cell_list *celllist;
    int i,nc,m_tot;

    celllist = xmalloc(sizeof(linked_cell_list));

    celllist->dimension = @D@;
    celllist->nx = (int)floor(geometry->l[0] / R_CUT);
    if(celllist->nx <= 0)
        celllist->nx = 1;
    celllist->ny = (int)floor(geometry->l[1] / R_CUT);
    if(celllist->ny <= 0)
        celllist->ny = 1;
    ifelse(@D@,3,
           celllist->nz = (int)floor(geometry->l[2] / R_CUT);
           if(celllist->nz <= 0)
               celllist->nz = 1;,
           celllist->nz = 1;)

    celllist->head = xmalloc(sizeof(int *) * particles->n);
    celllist->list = xmalloc(sizeof(int *) * particles->n);

    ifelse(@D@,2,
           nc = celllist->nx * celllist->ny;,
           @D@,3,
           nc = celllist->nx * celllist->ny * celllist->nz;)

    m_tot = total_number_of_soft_particles@D@(particles);

    celllist->head[0] = xmalloc(sizeof(int) * nc * particles->n);
    celllist->list[0] = xmalloc(sizeof(int) * m_tot);

    for(i = 0;i < particles->n;i++)
    {
        if(i > 0)
        {
            celllist->head[i] = celllist->head[i - 1] + nc;
            celllist->list[i] = celllist->list[i - 1] + particles->m[i - 1];
        }
    }

    clear_linked_cell_list@D@(celllist,particles);

    return celllist;
}

/*---------------------------------------------------------------------------*/
void clear_linked_cell_list@D@(linked_cell_list *celllist,
                               soft_particles@D@ *particles)
{
    int i,j,nc;

    ifelse(@D@,2,
           nc = celllist->nx * celllist->ny;,
           @D@,3,
           nc = celllist->nx * celllist->ny * celllist->nz;)

    for(i = 0;i < particles->n;i++)
    {
        for(j = 0;j < nc;j++)
        {
            celllist->head[i][j] = -1;
        }
        for(j = 0;j < particles->m[i];j++)
        {
            celllist->list[i][j] = -1;
        }
    }
}

/*---------------------------------------------------------------------------*/
void construct_linked_cell_list@D@(linked_cell_list *celllist,
                                 vector@D@ **r,
                                 soft_particles@D@ *particles,
                                 box_geometry *geometry)
{
    int i,j,nc;
    int index[@D@];
    int cell_index;
    vector@D@ c;

    c[0] = celllist->nx / geometry->l[0];
    c[1] = celllist->ny / geometry->l[1];
    ifelse(@D@,2,
           nc = celllist->nx * celllist->ny;,
           @D@,3,
           c[2] = celllist->nz / geometry->l[2];
           nc = celllist->nx * celllist->ny * celllist->nz;)

    clear_linked_cell_list@D@(celllist,particles);

    for(i = 0;i < particles->n;i++)
    {
        for(j = 0;j < particles->m[i];j++)
        {
                /*  calculate the index for the cell.  */
            VECTOR@D@(index[@@] = (int)(r[i][j][@@] * c[@@]);)

            ifelse(@D@,2,
                   cell_index = index[1] * celllist->nx + index[0];,
                   @D@,3,
                   cell_index = (index[2] * celllist->ny + index[1]) * celllist->nx + index[0];)

                /*  add the particle index into the list.  */
            celllist->list[i][j] = celllist->head[i][cell_index];
            celllist->head[i][cell_index] = j;
        }
    }
}'`')

/*---------------------------------------------------------------------------*/
