/*---------------------------------------------------------------------------*/

/*!  \file  celllist.h
     \brief  create or update the linked cell list.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_CELLLIST_H
#define INCLUDE_CELLLIST_H

#include "vector.h"
#include "particles.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  linked cell list for particles.  */
typedef struct linked_cell_list
{
        /*  \brief  dimension of the system.  */
    int dimension;

        /*  \brief  number of division.  */
    int nx,ny,nz;

        /*  \brief  head indexes of particles.  */
    int **head;
        /*  \brief  non-head indexes of particles.  */
    int **list;
} linked_cell_list;

/*---------------------------------------------------------------------------*/
/*!  \brief  allocate new linked cell list.
     \param  particles  soft particles
     \param  geometry  geometry of the simulation box  */
linked_cell_list *allocate_linked_cell_list(soft_particles *particles,
                                            box_geometry *geometry);
/*!  \brief  free a linked cell list.
     \param  celllist  linked cell list to be freed  */
void free_linked_cell_list(linked_cell_list *celllist);
/*!  \brief  construct a linked cell list from \a particles and \a geometry.
     \param  celllist  linked cell list
     \param  particles  soft particles
     \param  geometry  geometry of the simulation box  */
void construct_linked_cell_list(linked_cell_list *celllist,
                                soft_particles *particles,
                                box_geometry *geometry);
void construct_linked_cell_list_star(linked_cell_list *celllist,
                                     soft_particles *particles,
                                     box_geometry *geometry);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
