/*---------------------------------------------------------------------------*/

/*!  \file  geometry.c
     \brief  the geometry infomation for the box.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "xmalloc.h"
#include "geometry.h"

/*---------------------------------------------------------------------------*/
box_geometry *allocate_box_geometry(void)
{
    box_geometry *geometry;

    geometry = xmalloc(sizeof(box_geometry));

    return geometry;
}

/*---------------------------------------------------------------------------*/
void free_box_geometry(box_geometry *geometry)
{
    if(geometry == NULL)
        return;

    free(geometry);
}

/*---------------------------------------------------------------------------*/
double calculate_volume_of_box(box_geometry *geometry)
{
    double v;

    switch(geometry->dimension)
    {
        case 2:
            v = geometry->l[0] * geometry->l[1];
            break;

        case 3:
            v = geometry->l[0] * geometry->l[1] * geometry->l[2];
            break;

        default:
            v = 0.0;
            break;
    }

    return v;
}

/*---------------------------------------------------------------------------*/
void print_geometry(FILE *fp,box_geometry *geometry)
{
    switch(geometry->dimension)
    {
        case 2:
            fprintf(fp,"lattice: %g %g\n",geometry->l[0],geometry->l[1]);
            fprintf(fp,"delta_l: %g\n",geometry->delta_l);
            break;

        case 3:
            fprintf(fp,"lattice: %g %g %g\n",geometry->l[0],geometry->l[1],
                    geometry->l[2]);
            fprintf(fp,"delta_l: %g\n",geometry->delta_l);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
