/*---------------------------------------------------------------------------*/

/*!  \file  luautil.h
     \brief  some utility functions for Lua.  */

/*---------------------------------------------------------------------------*/

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from luautil.h.m4.  */

#ifndef INCLUDE_LUAUTIL_H
#define INCLUDE_LUAUTIL_H

#ifdef HAVE_LUA_H
#include <lua.h>
#endif
#ifdef HAVE_LUALIB_H
#include <lualib.h>
#endif
#ifdef HAVE_LAUXLIB_H
#include <lauxlib.h>
#endif

/*---------------------------------------------------------------------------*/
/*!  \brief  open \a filename as a Lua script.
     \param  lua_State *L  state of Lua
     \param  name  name of a table  */
lua_State *luau_open_file(const char *filename);

/*!  \brief  push \a name into the Lua stack as a table name.
     \param  lua_State *L  state of Lua
     \param  name  name of a table  */
int luau_push_table(lua_State *L,const char *name);
/*!  \brief  pop a table name from the Lua stack.
     \param  lua_State *L  state of Lua  */
int luau_pop_table(lua_State *L);
/*!  \brief  push \a name into the Lua stack as a table name (exit if an error is occurred).
     \param  lua_State *L  state of Lua
     \param  name  name of a table  */
int luau_push_table_or_exit(lua_State *L,const char *name);
/*!  \brief  pop a table name from the Lua stack (exit if an error is occurred).
     \param  lua_State *L  state of Lua  */
int luau_pop_table_or_exit(lua_State *L);

/*!  \brief  get int value of a variable \a name.
     \param  lua_State *L  state of Lua
     \param  name  name of a variable
     \param  value  value of a variable  */
int luau_get_int(lua_State *L,const char *name,int *value);
/*!  \brief  get bool value of a variable \a name.
     \param  lua_State *L  state of Lua
     \param  name  name of a variable
     \param  value  value of a variable  */
int luau_get_bool(lua_State *L,const char *name,int *value);
/*!  \brief  get unsigned_long value of a variable \a name.
     \param  lua_State *L  state of Lua
     \param  name  name of a variable
     \param  value  value of a variable  */
int luau_get_unsigned_long(lua_State *L,const char *name,unsigned long *value);
/*!  \brief  get double value of a variable \a name.
     \param  lua_State *L  state of Lua
     \param  name  name of a variable
     \param  value  value of a variable  */
int luau_get_double(lua_State *L,const char *name,double *value);
/*!  \brief  get string value of a variable \a name.
     \param  lua_State *L  state of Lua
     \param  name  name of a variable
     \param  value  value of a variable  */
int luau_get_string(lua_State *L,const char *name,char **value);
/*!  \brief  get int value of a variable \a name (exit if an error is occurred).
     \param  lua_State *L  state of Lua
     \param  name  name of a variable
     \param  value  value of a variable  */
int luau_get_int_or_exit(lua_State *L,const char *name,int *value);
/*!  \brief  get bool value of a variable \a name (exit if an error is occurred).
     \param  lua_State *L  state of Lua
     \param  name  name of a variable
     \param  value  value of a variable  */
int luau_get_bool_or_exit(lua_State *L,const char *name,int *value);
/*!  \brief  get unsigned_long value of a variable \a name (exit if an error is occurred).
     \param  lua_State *L  state of Lua
     \param  name  name of a variable
     \param  value  value of a variable  */
int luau_get_unsigned_long_or_exit(lua_State *L,const char *name,unsigned long *value);
/*!  \brief  get double value of a variable \a name (exit if an error is occurred).
     \param  lua_State *L  state of Lua
     \param  name  name of a variable
     \param  value  value of a variable  */
int luau_get_double_or_exit(lua_State *L,const char *name,double *value);
/*!  \brief  get string value of a variable \a name (exit if an error is occurred).
     \param  lua_State *L  state of Lua
     \param  name  name of a variable
     \param  value  value of a variable  */
int luau_get_string_or_exit(lua_State *L,const char *name,char **value);

/*!  \brief  get int value of a variable array \a name.
     \param  lua_State *L  state of Lua
     \param  name  name of a variable array
     \param  value  value of a variable array  */
int luau_get_int_array(lua_State *L,const char *name,int **value);
/*!  \brief  get double value of a variable array \a name.
     \param  lua_State *L  state of Lua
     \param  name  name of a variable array
     \param  value  value of a variable array  */
int luau_get_double_array(lua_State *L,const char *name,double **value);
/*!  \brief  get string value of a variable array \a name.
     \param  lua_State *L  state of Lua
     \param  name  name of a variable array
     \param  value  value of a variable array  */
int luau_get_string_array(lua_State *L,const char *name,char ***value);
/*!  \brief  get int value of a variable array \a name (exit if an error is occurred).
     \param  lua_State *L  state of Lua
     \param  name  name of a variable array
     \param  value  value of a variable array  */
int luau_get_int_array_or_exit(lua_State *L,const char *name,int **value);
/*!  \brief  get double value of a variable array \a name (exit if an error is occurred).
     \param  lua_State *L  state of Lua
     \param  name  name of a variable array
     \param  value  value of a variable array  */
int luau_get_double_array_or_exit(lua_State *L,const char *name,double **value);
/*!  \brief  get string value of a variable array \a name (exit if an error is occurred).
     \param  lua_State *L  state of Lua
     \param  name  name of a variable array
     \param  value  value of a variable array  */
int luau_get_string_array_or_exit(lua_State *L,const char *name,char ***value);

/*!  \brief  get int value of a variable 2d-array\a name.
     \param  lua_State *L  state of Lua
     \param  name  name of a variable 2d-array
     \param  value  value of a variable 2d-array  */
int *luau_get_int_array2d(lua_State *L,const char *name,int ***value);
/*!  \brief  get bool value of a variable 2d-array\a name.
     \param  lua_State *L  state of Lua
     \param  name  name of a variable 2d-array
     \param  value  value of a variable 2d-array  */
int *luau_get_bool_array2d(lua_State *L,const char *name,int ***value);
/*!  \brief  get double value of a variable 2d-array\a name.
     \param  lua_State *L  state of Lua
     \param  name  name of a variable 2d-array
     \param  value  value of a variable 2d-array  */
int *luau_get_double_array2d(lua_State *L,const char *name,double ***value);
/*!  \brief  get int value of a variable 2d-array\a name (exit if an error is occurred).
     \param  lua_State *L  state of Lua
     \param  name  name of a variable 2d-array
     \param  value  value of a variable 2d-array  */
int *luau_get_int_array2d_or_exit(lua_State *L,const char *name,int ***value);
/*!  \brief  get bool value of a variable 2d-array\a name (exit if an error is occurred).
     \param  lua_State *L  state of Lua
     \param  name  name of a variable 2d-array
     \param  value  value of a variable 2d-array  */
int *luau_get_bool_array2d_or_exit(lua_State *L,const char *name,int ***value);
/*!  \brief  get double value of a variable 2d-array\a name (exit if an error is occurred).
     \param  lua_State *L  state of Lua
     \param  name  name of a variable 2d-array
     \param  value  value of a variable 2d-array  */
int *luau_get_double_array2d_or_exit(lua_State *L,const char *name,double ***value);

/*!  \brief  free string allocated by \a luau_get_string.
     \param  value  string to free  */
void luau_free_string(char *value);

/*!  \brief  free int array allocated by \a luau_get_int_array.
     \param  value  int array to free  */
void luau_free_int_array(int *value);
/*!  \brief  free double array allocated by \a luau_get_double_array.
     \param  value  double array to free  */
void luau_free_double_array(double *value);
/*!  \brief  free string array allocated by \a luau_get_string_array.
     \param  value  string array to free  */
void luau_free_string_array(char **value);

/*!  \brief  free int 2d-array allocated by \a luau_get_int_array2d.
     \param  value  int 2d-array to free  */
void luau_free_int_array2d(int **value);
/*!  \brief  free bool 2d-array allocated by \a luau_get_bool_array2d.
     \param  value  int 2d-array to free  */
void luau_free_bool_array2d(int **value);
/*!  \brief  free double 2d-array allocated by \a luau_get_double_array2d.
     \param  value  double 2d-array to free  */
void luau_free_double_array2d(double **value);

/*---------------------------------------------------------------------------*/
#endif
