/*---------------------------------------------------------------------------*/

/*!  \file  particles.c
     \brief  the soft particles which represent polymers.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from particles.c.m4.  */

#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "xmalloc.h"
#include "vector.h"
#include "mt.h"
#include "particles.h"

/*---------------------------------------------------------------------------*/
static soft_particles *allocate_soft_particles2(polymer_blend *blend);
static void free_soft_particles2(soft_particles *particles);
static void set_particle_positions2_random(soft_particles2 *particles,
                                             polymer_blend *blend,
                                             box_geometry *geometry);
static void set_particle_positions2_into_box(soft_particles2 *particles,
                                               vector2 **r,
                                               box_geometry *geometry);

static soft_particles *allocate_soft_particles3(polymer_blend *blend);
static void free_soft_particles3(soft_particles *particles);
static void set_particle_positions3_random(soft_particles3 *particles,
                                             polymer_blend *blend,
                                             box_geometry *geometry);
static void set_particle_positions3_into_box(soft_particles3 *particles,
                                               vector3 **r,
                                               box_geometry *geometry);

/*---------------------------------------------------------------------------*/
soft_particles *allocate_soft_particles(polymer_blend *blend,
                                        box_geometry *geometry)
{
    soft_particles *particles;

        switch(geometry->dimension)
    {
        case 2:
            particles = allocate_soft_particles2(blend);
            break;

        case 3:
            particles = allocate_soft_particles3(blend);
            break;

        default:
            particles = NULL;
            break;
    }

    return particles;
}

/*---------------------------------------------------------------------------*/
void free_soft_particles(soft_particles *particles)
{
        switch(particles->dimension)
    {
        case 2:
            free_soft_particles2(particles);
            break;

        case 3:
            free_soft_particles3(particles);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void set_particle_positions_random(soft_particles *particles,
                                   polymer_blend *blend,
                                   box_geometry *geometry)
{
        switch(particles->dimension)
    {
        case 2:
            set_particle_positions2_random(&(particles->particles2),blend,geometry);
            break;

        case 3:
            set_particle_positions3_random(&(particles->particles3),blend,geometry);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void set_particle_positions_into_box(soft_particles *particles,
                                     box_geometry *geometry)
{
        switch(particles->dimension)
    {
        case 2:
            set_particle_positions2_into_box(&(particles->particles2),particles->particles2.r,geometry);
            break;

        case 3:
            set_particle_positions3_into_box(&(particles->particles3),particles->particles3.r,geometry);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void set_particle_positions_star_into_box(soft_particles *particles,
                                          box_geometry *geometry)
{
        switch(particles->dimension)
    {
        case 2:
            set_particle_positions2_into_box(&(particles->particles2),particles->particles2.r_star,geometry);
            break;

        case 3:
            set_particle_positions3_into_box(&(particles->particles3),particles->particles3.r_star,geometry);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
int total_number_of_soft_particles(soft_particles *particles)
{
    int m_tot;

        switch(particles->dimension)
    {
        case 2:
            m_tot = total_number_of_soft_particles2(&(particles->particles2));
            break;

        case 3:
            m_tot = total_number_of_soft_particles3(&(particles->particles3));
            break;

        default:
            m_tot = 0;
            break;
    }

    return m_tot;
}

/*---------------------------------------------------------------------------*/
soft_particles *allocate_soft_particles2(polymer_blend *blend)
{
    int i,m_tot;
    soft_particles *particles;
    soft_particles2 *particles2;

    particles = xmalloc(sizeof(soft_particles));
    particles->dimension = 2;

    particles2 = &(particles->particles2);
    particles2->n = blend->n;
    particles2->m = xmalloc(sizeof(int) * blend->n);
    particles2->r = xmalloc(sizeof(vector2 *) * blend->n);
    particles2->f = xmalloc(sizeof(vector2 *) * blend->n);
    particles2->w = xmalloc(sizeof(vector2 *) * blend->n);
    particles2->r_star = xmalloc(sizeof(vector2 *) * blend->n);
    particles2->f_star = xmalloc(sizeof(vector2 *) * blend->n);
    particles2->sigma = xmalloc(sizeof(symmetric_tensor2 *) * blend->n);

    m_tot = 0;
    for(i = 0;i < particles2->n;i++)
    {
        particles2->m[i] = blend->m[i];
        m_tot += particles2->m[i];
    }

    particles2->r[0] = xmalloc(sizeof(vector2) * m_tot);
    particles2->f[0] = xmalloc(sizeof(vector2) * m_tot);
    particles2->w[0] = xmalloc(sizeof(vector2) * m_tot);
    particles2->r_star[0] = xmalloc(sizeof(vector2) * m_tot);
    particles2->f_star[0] = xmalloc(sizeof(vector2) * m_tot);
    particles2->sigma[0] = xmalloc(sizeof(symmetric_tensor2) * m_tot);

    for(i = 0;i < (particles2->n - 1);i++)
    {
        particles2->r[i + 1] = particles2->r[i] + particles2->m[i];
        particles2->f[i + 1] = particles2->f[i] + particles2->m[i];
        particles2->w[i + 1] = particles2->w[i] + particles2->m[i];
        particles2->r_star[i + 1] = particles2->r_star[i] + particles2->m[i];
        particles2->f_star[i + 1] = particles2->f_star[i] + particles2->m[i];
        particles2->sigma[i + 1] = particles2->sigma[i] + particles2->m[i];
    }

    return particles;
}

/*---------------------------------------------------------------------------*/
void free_soft_particles2(soft_particles *particles)
{
    soft_particles2 *particles2;
    if(particles == NULL)
        return;

    particles2 = &(particles->particles2);

    free(particles2->r[0]);
    free(particles2->f[0]);
    free(particles2->w[0]);
    free(particles2->r_star[0]);
    free(particles2->f_star[0]);
    free(particles2->sigma[0]);
    free(particles2->r);
    free(particles2->f);
    free(particles2->w);
    free(particles2->r_star);
    free(particles2->f_star);
    free(particles2->sigma);
    free(particles2->m);
}

/*---------------------------------------------------------------------------*/
void set_particle_positions2_random(soft_particles2 *particles,
                                      polymer_blend *blend,
                                      box_geometry *geometry)
{
    int i,j;

    for(i = 0;i < particles->n;i++)
    {
        if(blend->next[i] < 0)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                particles->r[i][j][0] = geometry->l[0] * mtrand_real2();
particles->r[i][j][1] = geometry->l[1] * mtrand_real2();
            }
        }
    }

    for(i = 0;i < particles->n;i++)
    {
        if(blend->next[i] >= 0)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                particles->r[i][j][0] = particles->r[blend->next[i]][j][0];
particles->r[i][j][1] = particles->r[blend->next[i]][j][1];
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
void set_particle_positions2_into_box(soft_particles2 *particles,
                                        vector2 **r,
                                        box_geometry *geometry)
{
    int i,j;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(i = 0;i < particles->n;i++)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                while(r[i][j][0] < 0.0)
                              r[i][j][0] += geometry->l[0];
                          while(r[i][j][0] >= geometry->l[0])
                              r[i][j][0] -= geometry->l[0];
while(r[i][j][1] < 0.0)
                              r[i][j][1] += geometry->l[1];
                          while(r[i][j][1] >= geometry->l[1])
                              r[i][j][1] -= geometry->l[1];
            }
        }
    }
    else
    {
            /*  box deformation  */
        for(i = 0;i < particles->n;i++)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                while(r[i][j][1] < 0.0)
                {
                    r[i][j][1] += geometry->l[1];
                    r[i][j][0] += geometry->delta_l;
                }
                while(r[i][j][1] >= geometry->l[1])
                {
                    r[i][j][1] -= geometry->l[1];
                    r[i][j][0] -= geometry->delta_l;
                }
                while(r[i][j][0] < 0.0)
                    r[i][j][0] += geometry->l[0];
                while(r[i][j][0] >= geometry->l[0])
                    r[i][j][0] -= geometry->l[0];
                
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
int total_number_of_soft_particles2(soft_particles2 *particles)
{
    int i;
    int m_tot;

    m_tot = 0;
    for(i = 0;i < particles->n;i++)
    {
        m_tot += particles->m[i];
    }

    return m_tot;
}

/*---------------------------------------------------------------------------*/
soft_particles *allocate_soft_particles3(polymer_blend *blend)
{
    int i,m_tot;
    soft_particles *particles;
    soft_particles3 *particles3;

    particles = xmalloc(sizeof(soft_particles));
    particles->dimension = 3;

    particles3 = &(particles->particles3);
    particles3->n = blend->n;
    particles3->m = xmalloc(sizeof(int) * blend->n);
    particles3->r = xmalloc(sizeof(vector3 *) * blend->n);
    particles3->f = xmalloc(sizeof(vector3 *) * blend->n);
    particles3->w = xmalloc(sizeof(vector3 *) * blend->n);
    particles3->r_star = xmalloc(sizeof(vector3 *) * blend->n);
    particles3->f_star = xmalloc(sizeof(vector3 *) * blend->n);
    particles3->sigma = xmalloc(sizeof(symmetric_tensor3 *) * blend->n);

    m_tot = 0;
    for(i = 0;i < particles3->n;i++)
    {
        particles3->m[i] = blend->m[i];
        m_tot += particles3->m[i];
    }

    particles3->r[0] = xmalloc(sizeof(vector3) * m_tot);
    particles3->f[0] = xmalloc(sizeof(vector3) * m_tot);
    particles3->w[0] = xmalloc(sizeof(vector3) * m_tot);
    particles3->r_star[0] = xmalloc(sizeof(vector3) * m_tot);
    particles3->f_star[0] = xmalloc(sizeof(vector3) * m_tot);
    particles3->sigma[0] = xmalloc(sizeof(symmetric_tensor3) * m_tot);

    for(i = 0;i < (particles3->n - 1);i++)
    {
        particles3->r[i + 1] = particles3->r[i] + particles3->m[i];
        particles3->f[i + 1] = particles3->f[i] + particles3->m[i];
        particles3->w[i + 1] = particles3->w[i] + particles3->m[i];
        particles3->r_star[i + 1] = particles3->r_star[i] + particles3->m[i];
        particles3->f_star[i + 1] = particles3->f_star[i] + particles3->m[i];
        particles3->sigma[i + 1] = particles3->sigma[i] + particles3->m[i];
    }

    return particles;
}

/*---------------------------------------------------------------------------*/
void free_soft_particles3(soft_particles *particles)
{
    soft_particles3 *particles3;
    if(particles == NULL)
        return;

    particles3 = &(particles->particles3);

    free(particles3->r[0]);
    free(particles3->f[0]);
    free(particles3->w[0]);
    free(particles3->r_star[0]);
    free(particles3->f_star[0]);
    free(particles3->sigma[0]);
    free(particles3->r);
    free(particles3->f);
    free(particles3->w);
    free(particles3->r_star);
    free(particles3->f_star);
    free(particles3->sigma);
    free(particles3->m);
}

/*---------------------------------------------------------------------------*/
void set_particle_positions3_random(soft_particles3 *particles,
                                      polymer_blend *blend,
                                      box_geometry *geometry)
{
    int i,j;

    for(i = 0;i < particles->n;i++)
    {
        if(blend->next[i] < 0)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                particles->r[i][j][0] = geometry->l[0] * mtrand_real2();
particles->r[i][j][1] = geometry->l[1] * mtrand_real2();
particles->r[i][j][2] = geometry->l[2] * mtrand_real2();
            }
        }
    }

    for(i = 0;i < particles->n;i++)
    {
        if(blend->next[i] >= 0)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                particles->r[i][j][0] = particles->r[blend->next[i]][j][0];
particles->r[i][j][1] = particles->r[blend->next[i]][j][1];
particles->r[i][j][2] = particles->r[blend->next[i]][j][2];
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
void set_particle_positions3_into_box(soft_particles3 *particles,
                                        vector3 **r,
                                        box_geometry *geometry)
{
    int i,j;

    if(geometry->delta_l == 0.0)
    {
            /*  no box deformation  */
        for(i = 0;i < particles->n;i++)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                while(r[i][j][0] < 0.0)
                              r[i][j][0] += geometry->l[0];
                          while(r[i][j][0] >= geometry->l[0])
                              r[i][j][0] -= geometry->l[0];
while(r[i][j][1] < 0.0)
                              r[i][j][1] += geometry->l[1];
                          while(r[i][j][1] >= geometry->l[1])
                              r[i][j][1] -= geometry->l[1];
while(r[i][j][2] < 0.0)
                              r[i][j][2] += geometry->l[2];
                          while(r[i][j][2] >= geometry->l[2])
                              r[i][j][2] -= geometry->l[2];
            }
        }
    }
    else
    {
            /*  box deformation  */
        for(i = 0;i < particles->n;i++)
        {
            for(j = 0;j < particles->m[i];j++)
            {
                while(r[i][j][1] < 0.0)
                {
                    r[i][j][1] += geometry->l[1];
                    r[i][j][0] += geometry->delta_l;
                }
                while(r[i][j][1] >= geometry->l[1])
                {
                    r[i][j][1] -= geometry->l[1];
                    r[i][j][0] -= geometry->delta_l;
                }
                while(r[i][j][0] < 0.0)
                    r[i][j][0] += geometry->l[0];
                while(r[i][j][0] >= geometry->l[0])
                    r[i][j][0] -= geometry->l[0];
                while(r[i][j][2] < 0.0)
                           r[i][j][2] += geometry->l[2];
                       while(r[i][j][2] >= geometry->l[2])
                           r[i][j][2] -= geometry->l[2];
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
int total_number_of_soft_particles3(soft_particles3 *particles)
{
    int i;
    int m_tot;

    m_tot = 0;
    for(i = 0;i < particles->n;i++)
    {
        m_tot += particles->m[i];
    }

    return m_tot;
}

/*---------------------------------------------------------------------------*/
