/*---------------------------------------------------------------------------*/

/*!  \file  textwrite.c
     \brief  output the field data as ascii text .  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008,2018 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from textwrite.c.m4.  */

#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#ifdef HAVE_ZLIB_H
#include <zlib.h>
#endif

#include "particles.h"
#include "textwrite.h"

/*---------------------------------------------------------------------------*/
static void write_positions2_text(char *filename,soft_particles2 *particles);
static void write_forces2_text(char *filename,soft_particles2 *particles);
static void write_stresses2_text(char *filename,soft_particles2 *particles,
                                   box_geometry *geometry);
static void calculate_system_stress_tensor2(soft_particles2 *particles,
                                              box_geometry *geometry,
                                              symmetric_tensor2 sigma);

static void write_positions3_text(char *filename,soft_particles3 *particles);
static void write_forces3_text(char *filename,soft_particles3 *particles);
static void write_stresses3_text(char *filename,soft_particles3 *particles,
                                   box_geometry *geometry);
static void calculate_system_stress_tensor3(soft_particles3 *particles,
                                              box_geometry *geometry,
                                              symmetric_tensor3 sigma);

/*---------------------------------------------------------------------------*/
void write_positions_text(char *filename,soft_particles *particles)
{
        switch(particles->dimension)
    {
        case 2:
            write_positions2_text(filename,&(particles->particles2));
            break;

        case 3:
            write_positions3_text(filename,&(particles->particles3));
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void write_forces_text(char *filename,soft_particles *particles)
{
        switch(particles->dimension)
    {
        case 2:
            write_forces2_text(filename,&(particles->particles2));
            break;

        case 3:
            write_forces3_text(filename,&(particles->particles3));
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void write_stresses_text(char *filename,soft_particles *particles,
                         box_geometry *geometry)
{
        switch(particles->dimension)
    {
        case 2:
            write_stresses2_text(filename,&(particles->particles2),
                                           geometry);
            break;

        case 3:
            write_stresses3_text(filename,&(particles->particles3),
                                           geometry);
            break;

        default:
            break;
    }
}

/*---------------------------------------------------------------------------*/
void write_box_geometry_text(char *filename,box_geometry *geometry)
{
    FILE *fp;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = fopen(filename,"wb");

    switch(geometry->dimension)
    {
        case 2:
            fprintf(fp,"2\n");
            fprintf(fp,"%f %f\n",geometry->l[0],geometry->l[1]);
            fprintf(fp,"%f\n",geometry->delta_l);
            break;

        case 3:
            fprintf(fp,"3\n");
            fprintf(fp,"%f %f %f\n",geometry->l[0],geometry->l[1],geometry->l[2]);
            fprintf(fp,"%f\n",geometry->delta_l);
            break;

        default:
            break;
    }

    fclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_positions2_text(char *filename,soft_particles2 *particles)
{
    gzFile fp;
    int i,j;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = gzopen(filename,"wb");

    gzprintf(fp,"# 2 %d\n# ",particles->n);
    for(i = 0;i < particles->n;i++)
    {
        gzprintf(fp,"%d ",particles->m[i]);
    }
    gzprintf(fp,"\n\n");

    for(i = 0;i < particles->n;i++)
    {
        for(j = 0;j < particles->m[i];j++)
        {
            gzprintf(fp,"%f %f\n",particles->r[i][j][0],
                            particles->r[i][j][1]);
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_forces2_text(char *filename,soft_particles2 *particles)
{
    gzFile fp;
    int i,j;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = gzopen(filename,"wb");

    gzprintf(fp,"# 2 %d\n# ",particles->n);
    for(i = 0;i < particles->n;i++)
    {
        gzprintf(fp,"%d ",particles->m[i]);
    }
    gzprintf(fp,"\n\n");

    for(i = 0;i < particles->n;i++)
    {
        for(j = 0;j < particles->m[i];j++)
        {
            gzprintf(fp,"%f %f\n",particles->f[i][j][0],
                            particles->f[i][j][1]);
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_stresses2_text(char *filename,soft_particles2 *particles,
                            box_geometry *geometry)
{
    FILE *fp;
    symmetric_tensor2 sigma;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    calculate_system_stress_tensor2(particles,geometry,sigma);

    fp = fopen(filename,"wt");

    fprintf(fp,"%f %f %f\n",sigma[0],sigma[1],sigma[2]);

    fclose(fp);
}

/*---------------------------------------------------------------------------*/
void calculate_system_stress_tensor2(soft_particles2 *particles,
                                       box_geometry *geometry,
                                       symmetric_tensor2 sigma)
{
    int i,j;
    double v;

    sigma[0] = 0;
sigma[1] = 0;
sigma[2] = 0;

    for(i = 0;i < particles->n;i++)
    {
        for(j = 0;j < particles->m[i];j++)
        {
            sigma[0] += particles->sigma[i][j][0];
sigma[1] += particles->sigma[i][j][1];
sigma[2] += particles->sigma[i][j][2];
        }
    }

    v = calculate_volume_of_box(geometry);

    sigma[0] /= v;
sigma[1] /= v;
sigma[2] /= v;
}

/*---------------------------------------------------------------------------*/
void write_positions3_text(char *filename,soft_particles3 *particles)
{
    gzFile fp;
    int i,j;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = gzopen(filename,"wb");

    gzprintf(fp,"# 3 %d\n# ",particles->n);
    for(i = 0;i < particles->n;i++)
    {
        gzprintf(fp,"%d ",particles->m[i]);
    }
    gzprintf(fp,"\n\n");

    for(i = 0;i < particles->n;i++)
    {
        for(j = 0;j < particles->m[i];j++)
        {
            gzprintf(fp,"%f %f %f\n",particles->r[i][j][0],
                            particles->r[i][j][1],particles->r[i][j][2]);
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_forces3_text(char *filename,soft_particles3 *particles)
{
    gzFile fp;
    int i,j;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = gzopen(filename,"wb");

    gzprintf(fp,"# 3 %d\n# ",particles->n);
    for(i = 0;i < particles->n;i++)
    {
        gzprintf(fp,"%d ",particles->m[i]);
    }
    gzprintf(fp,"\n\n");

    for(i = 0;i < particles->n;i++)
    {
        for(j = 0;j < particles->m[i];j++)
        {
            gzprintf(fp,"%f %f %f\n",particles->f[i][j][0],
                            particles->f[i][j][1],particles->f[i][j][2]);
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_stresses3_text(char *filename,soft_particles3 *particles,
                            box_geometry *geometry)
{
    FILE *fp;
    symmetric_tensor3 sigma;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    calculate_system_stress_tensor3(particles,geometry,sigma);

    fp = fopen(filename,"wt");

    fprintf(fp,"%f %f %f %f %f %f\n",sigma[0],sigma[1],
                   sigma[2],sigma[3],sigma[4],sigma[5]);

    fclose(fp);
}

/*---------------------------------------------------------------------------*/
void calculate_system_stress_tensor3(soft_particles3 *particles,
                                       box_geometry *geometry,
                                       symmetric_tensor3 sigma)
{
    int i,j;
    double v;

    sigma[0] = 0;
sigma[1] = 0;
sigma[2] = 0;
sigma[3] = 0;
sigma[4] = 0;
sigma[5] = 0;

    for(i = 0;i < particles->n;i++)
    {
        for(j = 0;j < particles->m[i];j++)
        {
            sigma[0] += particles->sigma[i][j][0];
sigma[1] += particles->sigma[i][j][1];
sigma[2] += particles->sigma[i][j][2];
sigma[3] += particles->sigma[i][j][3];
sigma[4] += particles->sigma[i][j][4];
sigma[5] += particles->sigma[i][j][5];
        }
    }

    v = calculate_volume_of_box(geometry);

    sigma[0] /= v;
sigma[1] /= v;
sigma[2] /= v;
sigma[3] /= v;
sigma[4] /= v;
sigma[5] /= v;
}

/*---------------------------------------------------------------------------*/
