/*---------------------------------------------------------------------------*/

/*!  \file  bond.c
     \brief  calculate the bond vectors and lengths.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2008 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "xmalloc.h"
#include "bond.h"

/*---------------------------------------------------------------------------*/
int calulate_bond(double **r,double **u,int dimension,int n,int *m,
                  double *l,double delta_l)
{
    int i,j;

    if((n != 2) || (m[0] != m[1]))
    {
        return EXIT_FAILURE;
    }

    (*u) = xmalloc(sizeof(double) * m[0] * dimension);

    for(i = 0;i < m[0];i++)
    {
        for(j = 0;j < dimension;j++)
        {
            (*u)[i * dimension + j] =
                r[1][i * dimension + j]
                - r[0][i * dimension + j];
        }

        while((*u)[i * dimension + 1] >= 0.5 * l[1])
        {
            (*u)[i * dimension + 1] -= l[1];
            (*u)[i * dimension + 0] -= delta_l;
        }
        while((*u)[i * dimension + 1] < - 0.5 * l[1])
        {
            (*u)[i * dimension + 1] += l[1];
            (*u)[i * dimension + 0] += delta_l;
        }

        while((*u)[i * dimension + 0] >= 0.5 * l[0])
            (*u)[i * dimension + 0] -= l[0];
        while((*u)[i * dimension + 0] < -0.5 * l[0])
            (*u)[i * dimension + 0] += l[0];

        if(dimension == 3)
        {
            while((*u)[i * dimension + 2] >= 0.5 * l[2])
                (*u)[i * dimension + 2] -= l[2];
            while((*u)[i * dimension + 2] < -0.5 * l[2])
                (*u)[i * dimension + 2] += l[2];
        }
    }

    return EXIT_SUCCESS;
}

/*---------------------------------------------------------------------------*/
