/*---------------------------------------------------------------------------*/

/*!  \file  dxread.h
     \brief  read field data from DX file generated by spreads.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_DXREAD_H
#define INCLUDE_DXREAD_H

/*---------------------------------------------------------------------------*/
/*!  \brief  read the positional data from \a filename (DX format).
     \param  filename  file name
     \param  r  positions of soft particles
     \param  dimension  dimension of the system
     \param  n  number of components
     \param  m  number of soft particles  */
int read_positions_dx(char *filename,double ***r,int *dimension,
                      int *n,int **m);
/*!  \brief  read the geometry data from \a filename (DX format).
     \param  filename  file name
     \param  dimension  dimension of the system
     \param  l  edge lengthes for the simulation box
     \param  delta_l  gap between cells by shear deformation  */
int read_geometry_dx(char *filename,int *dimension,double *l,double *delta_l);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
