/*---------------------------------------------------------------------------*/

/*!  \file  textwrite.c
     \brief  output the position data as gzipped ascii text format.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2018 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#ifdef HAVE_ZLIB_H
#include <zlib.h>
#endif

#include "textwrite.h"

/*---------------------------------------------------------------------------*/
void write_positions_text(char *filename,double **r,int dimension,
                          int n,int *m)
{
    gzFile fp;
    int i,j,k;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = gzopen(filename,"wb");

    gzprintf(fp,"# %d %d\n# ",dimension,n);
    for(i = 0;i < n;i++)
    {
        gzprintf(fp,"%d ",m[i]);
    }
    gzprintf(fp,"\n\n");

    for(i = 0;i < n;i++)
    {
        for(j = 0;j < m[i];j++)
        {
            for(k = 0;k < dimension;k++)
            {
                gzprintf(fp,"%f%c",r[i][j * dimension + k],
                         ((k == (dimension - 1)) ? '\n' : ' '));
            }
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_geometry_text(char *filename,int dimension,double *l,
                         double delta_l)
{
    FILE *fp;
    int i;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = fopen(filename,"wt");

    fprintf(fp,"%d\n",dimension);

    fprintf(fp,"%f",l[0]);
    for(i = 1;i < dimension;i++)
        fprintf(fp," %f",l[i]);
    fprintf(fp,"\n");

    fprintf(fp,"%f\n",delta_l);

    fclose(fp);
}

/*---------------------------------------------------------------------------*/
void write_bond_text(char *filename,double *u,int dimension,int m)
{
    gzFile fp;
    int i,j;

    if((filename == NULL) || (strlen(filename) == 0))
        return;

    fp = gzopen(filename,"wb");

    gzprintf(fp,"# %d %d\n",dimension,m);

    for(i = 0;i < m;i++)
    {
        gzprintf(fp,"%f",u[i * dimension]);
        for(j = 1;j < dimension;j++)
        {
            gzprintf(fp," %f",u[i * dimension + j]);
        }
        gzprintf(fp,"\n");
    }

    gzclose(fp);
}

/*---------------------------------------------------------------------------*/
